/* Information on characters in the JIS X 0208 two-byte encoding
   (containing Kanji, Kana, Roman letters, Greek letters, Cyrillic letters,
   etc).  This file is #included by g_cntrlify.c. */

/* Whether a JIS index (two bytes) defines a JIS character, irrespective of
   whether we have a glyph for it.  (If it isn't a character, it ideally
   prints as a zero-width glyph; if it's a character but we don't have it,
   it ideally prints as an empty glyph or an `undefined' glyph.) */
   
#define GOOD_JIS_INDEX(row, col) \
	((row) > 0x20 && (row) < 0x7f && (col) > 0x20 && (col) < 0x7f)

#define BEGINNING_OF_KANJI 0x3000 /* Kanji afterwards, non-Kanji before */

#ifndef RAW_HERSHEY_GLYPH
#define RAW_HERSHEY_GLYPH 0x4000
#endif

struct kanjipair
{
  int jis, nelson;
};

/* The 603 Kanji that we have glyphs for.  1st field is JIS index number;
   2nd is Nelson index number, which we use internally (it indexes into the
   Hershey oriental glyph array in g_her_glyph.c).  For info on
   correspondence, see kanji.doc. */

const struct kanjipair _builtin_kanji_glyphs[] =
{
  {0x3021, 43},
  {0x3026, 2829},
  {0x302d, 62},
  {0x3035, 818},
  {0x303f, 1802},
  {0x3045, 2154},
  {0x304c, 401},
  {0x3057, 2107},
  {0x3059, 138},
  {0x305b, 3008},
  {0x305e, 3579},
  {0x3061, 4214},
  {0x306c, 1},
  {0x3070, 3294},
  {0x3078, 1026},
  {0x307a, 1562},
  {0x3122, 5006},
  {0x3126, 878},
  {0x3127, 1280},
  {0x3129, 3673},
  {0x312b, 5042},
  {0x3132, 2629},
  {0x313b, 2973},
  {0x313f, 4725},
  {0x3140, 5046},
  {0x314a, 130},
  {0x3155, 2599},
  {0x315f, 617},
  {0x3173, 4733},
  {0x3176, 1125},
  {0x3177, 2083},
  {0x317e, 1504},
  {0x3221, 1885},
  {0x3223, 2361},
  {0x3226, 2922},
  {0x322b, 5399},
  {0x322f, 551},
  {0x3235, 260},
  {0x3239, 2634},
  {0x323b, 5110},
  {0x323c, 9},
  {0x323d, 350},
  {0x323f, 409},
  {0x3241, 422},
  {0x3243, 716},
  {0x3244, 24},
  {0x3246, 58},
  {0x3248, 1311},
  {0x324a, 3272},
  {0x324c, 107},
  {0x324f, 2530},
  {0x3250, 2743},
  {0x3256, 3909},
  {0x3259, 3956},
  {0x3261, 4723},
  {0x3267, 2848},
  {0x3268, 50},
  {0x3272, 4306},
  {0x3273, 1028},
  {0x3323, 2264},
  {0x3324, 2553},
  {0x3326, 2998},
  {0x3328, 3537},
  {0x332b, 4950},
  {0x332d, 4486},
  {0x3330, 1168},
  {0x3346, 1163},
  {0x334b, 2254},
  {0x3351, 4301},
  {0x3353, 4623},
  {0x3357, 5088},
  {0x3358, 1271},
  {0x335a, 2324},
  {0x3364, 703},
  {0x3424, 2977},
  {0x3428, 1322},
  {0x342c, 1466},
  {0x3433, 1492},
  {0x3434, 790},
  {0x3436, 1731},
  {0x3437, 1756},
  {0x3445, 2988},
  {0x3449, 3416},
  {0x3454, 4750},
  {0x3456, 4949},
  {0x3458, 4958},
  {0x346f, 994},
  {0x3470, 1098},
  {0x3476, 1496},
  {0x347c, 3785},
  {0x3521, 2379},
  {0x3522, 1582},
  {0x3524, 2480},
  {0x3525, 2507},
  {0x352d, 4318},
  {0x3530, 4610},
  {0x3534, 5276},
  {0x3535, 5445},
  {0x3546, 3981},
  {0x3555, 4685},
  {0x355a, 154},
  {0x355b, 885},
  {0x355d, 1560},
  {0x3565, 2941},
  {0x3566, 3314},
  {0x3569, 3496},
  {0x356d, 2852},
  {0x356e, 1051},
  {0x356f, 1387},
  {0x3575, 4109},
  {0x3577, 4548},
  {0x357b, 5281},
  {0x357e, 295},
  {0x3621, 431},
  {0x3626, 581},
  {0x362d, 1135},
  {0x362f, 1571},
  {0x3635, 2052},
  {0x3636, 2378},
  {0x364a, 103},
  {0x364b, 2305},
  {0x364c, 2923},
  {0x3651, 1065},
  {0x3661, 4671},
  {0x3662, 4815},
  {0x3664, 4855},
  {0x3665, 146},
  {0x3671, 3128},
  {0x3675, 3317},
  {0x367e, 1386},
  {0x3738, 449},
  {0x3739, 534},
  {0x373e, 2937},
  {0x373f, 1077},
  {0x3741, 1589},
  {0x3742, 1602},
  {0x374f, 195},
  {0x3750, 3523},
  {0x3757, 4312},
  {0x375a, 4620},
  {0x3767, 2412},
  {0x3768, 2509},
  {0x376a, 3313},
  {0x376b, 3540},
  {0x376c, 4205},
  {0x376e, 2169},
  {0x3777, 1045},
  {0x3824, 2868},
  {0x3826, 3180},
  {0x3828, 3543},
  {0x382b, 4284},
  {0x3833, 5220},
  {0x3835, 275},
  {0x3836, 825},
  {0x3839, 1568},
  {0x383a, 2637},
  {0x383b, 2656},
  {0x383d, 2943},
  {0x3840, 4309},
  {0x3842, 4987},
  {0x3845, 770},
  {0x3847, 1036},
  {0x384c, 1567},
  {0x384d, 1817},
  {0x384e, 2044},
  {0x385d, 5415},
  {0x385e, 15},
  {0x3861, 162},
  {0x3865, 1610},
  {0x3866, 1628},
  {0x386c, 4374},
  {0x3872, 290},
  {0x3877, 1358},
  {0x3878, 579},
  {0x387d, 868},
  {0x387e, 101},
  {0x3929, 1451},
  {0x3931, 1683},
  {0x393d, 2343},
  {0x3943, 92},
  {0x394d, 3684},
  {0x3954, 4213},
  {0x3955, 1641},
  {0x395b, 4843},
  {0x395d, 4883},
  {0x395f, 4994},
  {0x3960, 1459},
  {0x3961, 5188},
  {0x3962, 5248},
  {0x3966, 882},
  {0x3967, 383},
  {0x3971, 1037},
  {0x3975, 5403},
  {0x397c, 5236},
  {0x397e, 4660},
  {0x3a21, 2430},
  {0x3a23, 352},
  {0x3a2c, 2261},
  {0x3a38, 1455},
  {0x3a39, 3662},
  {0x3a42, 1515},
  {0x3a46, 35},
  {0x3a47, 2146},
  {0x3a59, 3522},
  {0x3a5f, 1055},
  {0x3a6e, 407},
  {0x3a72, 2119},
  {0x3a79, 2256},
  {0x3b2e, 3113},
  {0x3b30, 8},
  {0x3b33, 1407},
  {0x3b36, 2056},
  {0x3b3b, 3415},
  {0x3b40, 4789},
  {0x3b45, 362},
  {0x3b4d, 1025},
  {0x3b4e, 1160},
  {0x3b4f, 1208},
  {0x3b52, 1264},
  {0x3b54, 284},
  {0x3b57, 3001},
  {0x3b58, 1904},
  {0x3b59, 2039},
  {0x3b5e, 2211},
  {0x3b5f, 2429},
  {0x3b60, 2439},
  {0x3b61, 2478},
  {0x3b64, 3265},
  {0x3b65, 3492},
  {0x3b66, 3510},
  {0x3b6a, 3845},
  {0x3b73, 2435},
  {0x3b75, 5428},
  {0x3b76, 272},
  {0x3b7a, 1281},
  {0x3b7d, 1903},
  {0x3b7e, 2126},
  {0x3c21, 638},
  {0x3c27, 3209},
  {0x3c28, 3228},
  {0x3c2a, 3697},
  {0x3c2b, 3841},
  {0x3c2d, 3860},
  {0x3c2f, 5375},
  {0x3c30, 1556},
  {0x3c34, 4619},
  {0x3c37, 261},
  {0x3c3c, 1300},
  {0x3c3e, 2631},
  {0x3c41, 4518},
  {0x3c42, 1297},
  {0x3c4d, 4603},
  {0x3c50, 2074},
  {0x3c54, 3685},
  {0x3c56, 4608},
  {0x3c5c, 1377},
  {0x3c61, 4809},
  {0x3c63, 3926},
  {0x3c67, 285},
  {0x3c68, 3699},
  {0x3c6a, 1827},
  {0x3c6f, 3295},
  {0x3c72, 2573},
  {0x3c73, 5186},
  {0x3c7e, 622},
  {0x3d29, 3273},
  {0x3d2a, 3521},
  {0x3d2e, 3863},
  {0x3d39, 4798},
  {0x3d3d, 768},
  {0x3d3e, 1613},
  {0x3d44, 3597},
  {0x3d45, 224},
  {0x3d50, 97},
  {0x3d51, 1621},
  {0x3d55, 2122},
  {0x3d60, 791},
  {0x3d63, 3509},
  {0x3d68, 1162},
  {0x3d6b, 2138},
  {0x3d71, 3719},
  {0x3d77, 1185},
  {0x3d7c, 4993},
  {0x3e26, 321},
  {0x3e2e, 1355},
  {0x3e2f, 166},
  {0x3e3d, 2137},
  {0x3e3e, 2212},
  {0x3e46, 2772},
  {0x3e4b, 3192},
  {0x3e4e, 3280},
  {0x3e57, 1638},
  {0x3e5a, 4341},
  {0x3e5d, 4472},
  {0x3e65, 798},
  {0x3e68, 223},
  {0x3e6c, 1113},
  {0x3e6f, 1364},
  {0x3e75, 2839},
  {0x3e78, 4002},
  {0x3f22, 2303},
  {0x3f27, 3889},
  {0x3f29, 5154},
  {0x3f2d, 403},
  {0x3f34, 1645},
  {0x3f36, 1920},
  {0x3f37, 2080},
  {0x3f39, 2301},
  {0x3f3f, 783},
  {0x3f43, 3837},
  {0x3f48, 4601},
  {0x3f49, 4646},
  {0x3f4a, 4709},
  {0x3f4c, 5055},
  {0x3f4d, 339},
  {0x3f5e, 1034},
  {0x3f62, 211},
  {0x3f65, 2482},
  {0x3f69, 3676},
  {0x3f74, 2057},
  {0x402d, 1666},
  {0x402e, 1799},
  {0x4030, 2436},
  {0x4031, 2121},
  {0x4032, 2143},
  {0x4035, 27},
  {0x4038, 2991},
  {0x403e, 4273},
  {0x4044, 5076},
  {0x4045, 5077},
  {0x404e, 2108},
  {0x404f, 2194},
  {0x4050, 3176},
  {0x4051, 3306},
  {0x4056, 4534},
  {0x405a, 667},
  {0x405c, 1951},
  {0x405e, 1855},
  {0x4063, 5044},
  {0x4064, 3539},
  {0x4065, 3855},
  {0x4068, 571},
  {0x4069, 156},
  {0x406e, 1447},
  {0x4070, 1823},
  {0x407e, 3580},
  {0x4125, 3873},
  {0x4130, 595},
  {0x4133, 2770},
  {0x4134, 384},
  {0x4147, 3511},
  {0x4148, 3520},
  {0x4150, 859},
  {0x4158, 1402},
  {0x415b, 1728},
  {0x4161, 2100},
  {0x416a, 2241},
  {0x416d, 3567},
  {0x4170, 3939},
  {0x4175, 4234},
  {0x4176, 4539},
  {0x417c, 540},
  {0x417d, 1137},
  {0x4224, 4701},
  {0x4226, 509},
  {0x422b, 196},
  {0x422c, 2632},
  {0x422d, 4546},
  {0x422e, 4700},
  {0x4233, 3544},
  {0x4236, 590},
  {0x4238, 1267},
  {0x423e, 361},
  {0x423f, 1169},
  {0x4240, 1172},
  {0x424a, 2313},
  {0x424e, 405},
  {0x4250, 2067},
  {0x4256, 1743},
  {0x4265, 364},
  {0x4267, 1171},
  {0x4268, 3385},
  {0x426a, 2164},
  {0x426c, 2655},
  {0x4274, 2503},
  {0x4323, 4721},
  {0x432b, 4458},
  {0x432f, 4384},
  {0x4331, 139},
  {0x433a, 1418},
  {0x433b, 3172},
  {0x4346, 1575},
  {0x434b, 2996},
  {0x434d, 488},
  {0x434e, 3169},
  {0x434f, 1056},
  {0x4356, 3644},
  {0x4359, 4722},
  {0x435d, 3366},
  {0x4362, 3325},
  {0x4363, 3940},
  {0x4365, 3665},
  {0x4366, 81},
  {0x4368, 1291},
  {0x436b, 53},
  {0x436c, 2236},
  {0x436e, 4115},
  {0x442b, 3788},
  {0x442c, 2702},
  {0x4436, 4543},
  {0x4439, 4938},
  {0x443b, 5340},
  {0x443e, 775},
  {0x444c, 4703},
  {0x4463, 406},
  {0x446a, 1296},
  {0x446c, 1508},
  {0x446d, 1514},
  {0x4472, 1914},
  {0x4478, 3285},
  {0x4479, 3581},
  {0x4526, 1987},
  {0x452a, 3097},
  {0x452f, 931},
  {0x4534, 4844},
  {0x4535, 588},
  {0x4537, 16},
  {0x453e, 4615},
  {0x4540, 804},
  {0x4544, 2994},
  {0x4545, 5050},
  {0x454c, 1614},
  {0x4559, 1511},
  {0x455a, 1050},
  {0x455f, 1161},
  {0x4561, 665},
  {0x4563, 1109},
  {0x4567, 230},
  {0x456c, 213},
  {0x4574, 2745},
  {0x4576, 1359},
  {0x4579, 3396},
  {0x4626, 4465},
  {0x4630, 730},
  {0x4631, 619},
  {0x4633, 1354},
  {0x463b, 4724},
  {0x463c, 4853},
  {0x4643, 2860},
  {0x4649, 4375},
  {0x465e, 2160},
  {0x4662, 82},
  {0x466e, 778},
  {0x4671, 5038},
  {0x4673, 273},
  {0x4679, 3724},
  {0x467c, 2097},
  {0x467e, 574},
  {0x4721, 1189},
  {0x472e, 2797},
  {0x472f, 188},
  {0x4733, 2808},
  {0x4734, 3472},
  {0x4748, 2529},
  {0x474f, 5191},
  {0x4769, 3275},
  {0x4772, 3095},
  {0x477e, 5385},
  {0x4821, 49},
  {0x482c, 577},
  {0x482f, 3092},
  {0x483e, 132},
  {0x483f, 817},
  {0x4841, 1469},
  {0x484c, 3865},
  {0x4856, 4811},
  {0x4860, 1604},
  {0x4866, 2470},
  {0x4869, 3109},
  {0x4873, 5080},
  {0x4874, 5152},
  {0x4878, 1383},
  {0x4879, 1631},
  {0x487e, 3658},
  {0x4921, 5421},
  {0x492e, 3397},
  {0x4934, 33},
  {0x4938, 2359},
  {0x4939, 131},
  {0x493d, 108},
  {0x4942, 3042},
  {0x4943, 3271},
  {0x494a, 923},
  {0x4954, 17},
  {0x495b, 1468},
  {0x4963, 2832},
  {0x4969, 4488},
  {0x4977, 5148},
  {0x497d, 1484},
  {0x4a23, 4255},
  {0x4a26, 173},
  {0x4a2a, 2857},
  {0x4a2c, 578},
  {0x4a38, 2064},
  {0x4a39, 4959},
  {0x4a3f, 26},
  {0x4a42, 589},
  {0x4a44, 4945},
  {0x4a46, 3461},
  {0x4a50, 511},
  {0x4a51, 306},
  {0x4a52, 2842},
  {0x4a55, 4661},
  {0x4a6c, 2466},
  {0x4a7c, 2084},
  {0x4a7d, 2082},
  {0x4b21, 2535},
  {0x4b26, 3749},
  {0x4b4c, 751},
  {0x4b4f, 5404},
  {0x4b5c, 96},
  {0x4b63, 5390},
  {0x4b68, 2467},
  {0x4b74, 855},
  {0x4b7c, 7},
  {0x4c23, 913},
  {0x4c24, 179},
  {0x4c29, 1316},
  {0x4c35, 2773},
  {0x4c37, 3164},
  {0x4c3e, 1170},
  {0x4c40, 2110},
  {0x4c4c, 5087},
  {0x4c53, 2473},
  {0x4c5a, 2170},
  {0x4c5c, 3127},
  {0x4c64, 4944},
  {0x4c67, 4940},
  {0x4c6b, 298},
  {0x4c70, 3168},
  {0x4c72, 1598},
  {0x4c74, 4074},
  {0x4c78, 2233},
  {0x4c7d, 2534},
  {0x4d2d, 3727},
  {0x4d30, 2565},
  {0x4d3a, 5030},
  {0x4d3c, 1167},
  {0x4d3e, 408},
  {0x4d4f, 2659},
  {0x4d51, 2993},
  {0x4d53, 3656},
  {0x4d55, 4001},
  {0x4d57, 4274},
  {0x4d5b, 5012},
  {0x4d63, 3680},
  {0x4d68, 202},
  {0x4d6b, 5049},
  {0x4d70, 3856},
  {0x4d71, 199},
  {0x4d72, 1431},
  {0x4d78, 3264},
  {0x4d7d, 2942},
  {0x4e24, 4813},
  {0x4e25, 5040},
  {0x4e26, 5005},
  {0x4e28, 319},
  {0x4e29, 3343},
  {0x4e2e, 2576},
  {0x4e32, 3191},
  {0x4e33, 3471},
  {0x4e35, 5440},
  {0x4e3e, 34},
  {0x4e41, 3468},
  {0x4e49, 3885},
  {0x4e4c, 2141},
  {0x4e4f, 715},
  {0x4e53, 2210},
  {0x4e55, 2807},
  {0x4e58, 4630},
  {0x4e60, 5138},
  {0x4e63, 428},
  {0x4e64, 642},
  {0x4e6d, 5048},
  {0x4e6e, 5056},
  {0x4e73, 2438},
  {0x4f22, 4702},
  {0x4f27, 2750},
  {0x4f29, 4561},
  {0x4f37, 3683},
  {0x4f3b, 283},
  {0x4f40, 4391},
  {0x4f42, 3268},
  {0x4f43, 4358},
  {0x4f44, 54},
  {0x4f47, 1710},
  {0x534c, 973},
  {0x5879, 1794},
  {0x5960, 1942},
  {0x626f, 3200},
  {0x6446, 3458},
  {0x6647, 5083},
  {0x6d55, 4633},
  {0, 0}
};

/* Non-Kanji characters in the JIS character set.  We map nearly all of
   these into characters (in the 0..255 range) in our existing fonts, which
   are laid out in g_fontdb.c.  The RAW_HERSHEY_GLYPH bit is set, if the
   character is to be mapped directly to a Hershey glyph (presumably one
   that appears in no font). */

struct jis_entry
{
  int jis, font;
  unsigned short charnum;
};

const struct jis_entry _builtin_jis_chars[] =
{
/* Row 1 [misc.] */
  {0x2121, 0, ' '},
  {0x2122, 14, 0x79},
  {0x2123, 14, 0x7a},
  {0x2124, 0, ','},
  {0x2125, 0, '.'},
  {0x2126, 0, 0xb7},
  {0x2127, 0, ':'},
  {0x2128, 0, ';'},
  {0x2129, 0, '?'},
  {0x212a, 0, '!'},
  /* 0x212b..0x212c are Japanese diacritics, shifted left */
  {0x212d, 0, RAW_HERSHEY_GLYPH + 4180},
  {0x212e, 0, RAW_HERSHEY_GLYPH + 4181},
  {0x212f, 0, RAW_HERSHEY_GLYPH + 4182},
  {0x2130, 0, RAW_HERSHEY_GLYPH + 4184},
  {0x2131, 0, RAW_HERSHEY_GLYPH + 4008},
  {0x2132, 0, RAW_HERSHEY_GLYPH + 4012},
  /* 0x2133..0x213a are Japanese characters */
  {0x213b, 0, RAW_HERSHEY_GLYPH + 903},
  {0x213c, 15, 0x78},
  {0x213d, 0, '-'},
  {0x213e, 0, 0xad},
  {0x213f, 0, '/'},
  {0x2140, 0, '\\'},
  {0x2141, 0, '~'},
  {0x2142, 0, RAW_HERSHEY_GLYPH + 2230},
  {0x2143, 0, '|'},
  {0x2144, 0, RAW_HERSHEY_GLYPH + 276},
  {0x2145, 0, RAW_HERSHEY_GLYPH + 238},
  {0x2146, 0, '`'},
  {0x2147, 0, '\''},
  {0x2148, 0, '`'},		/* should be double quotes, fixme */
  {0x2149, 0, '\''},		/* same */
  {0x214a, 0, '('},
  {0x214b, 0, ')'},
  {0x214c, 0, '['},		/* should be `bent' brackets, fixme */
  {0x214d, 0, ']'},		/* same */
  {0x214e, 0, '['},
  {0x214f, 0, ']'},
  {0x2150, 0, '{'},
  {0x2151, 0, '}'},
  {0x2152, 17, 0xe1},
  {0x2153, 17, 0xf1},
  /* 0x2154..0x2155 are double angle brackets */
  {0x2156, 17, 0xe9},
  {0x2157, 17, 0xfb},
  {0x2158, 17, 0xe9},		/* should be outlined \lc and \rf, fixme */
  {0x2159, 17, 0xfb},		/* same */
  {0x215a, 2, '['},		/* shouldn't be ordinary bold brackets, fixme */
  {0x215b, 2, ']'},		/* same */
  {0x215c, 0, '+'},
  {0x215d, 0, '-'},
  {0x215e, 17, 0xb1},
  {0x215f, 0, 0xd7},
  {0x2160, 0, 0xf7},
  {0x2161, 0, '='},
  {0x2162, 17, 0xb9},
  {0x2163, 0, '<'},
  {0x2164, 0, '>'},
  {0x2165, 17, 0xa3},
  {0x2166, 17, 0xb3},
  {0x2167, 17, 0xa5},
  {0x2168, 17, 0x5c},
  {0x2169, 0, RAW_HERSHEY_GLYPH + 2285},
  {0x216a, 0, RAW_HERSHEY_GLYPH + 2283},
  {0x216b, 17, 0xb0},
  {0x216c, 17, 0xa2},
  {0x216d, 17, 0xb2},
  /* 0x216e is degrees centigrade */
  {0x216f, 0, 0xa5},
  {0x2170, 0, '$'},
  {0x2171, 0, 0xa2},
  {0x2172, 0, 0xa3},
  {0x2173, 0, '%'},
  {0x2174, 0, '#'},
  {0x2175, 0, '&'},
  {0x2176, 0, '*'},
  {0x2177, 0, '@'},
  {0x2178, 0, 0xa7},
  /* 0x2179..0x217a are 5-pointed stars, unfilled and filled */
  {0x217b, 0, RAW_HERSHEY_GLYPH + 904},
  {0x217c, 0, RAW_HERSHEY_GLYPH + 904},	/* should be filled circle */
  {0x217d, 0, RAW_HERSHEY_GLYPH + 4109},
  {0x217e, 17, 0xe0},		/* lozenge rather than square; OK? */
/* Row 2 [misc.] */
  /* 0x2220..0x2229 are mostly squares, triangles */
  {0x222a, 17, 0xae},
  {0x222b, 17, 0xac},
  {0x222c, 17, 0xad},
  {0x222d, 17, 0xaf},
  {0x222d, 2, '='},	/* 0x222e is two bold horizontal strokes, NOT bold = */
  {0x223a, 17, 0xce},
  {0x223b, 17, RAW_HERSHEY_GLYPH + 282},
  {0x223c, 17, 0xcd},
  {0x223d, 17, 0xca},
  {0x223e, 17, 0xcc},
  {0x223f, 17, 0xc9},
  {0x2240, 17, 0xc8},
  {0x2241, 17, 0xc7},
  {0x224a, 17, 0xd9},
  {0x224b, 17, 0xda},
  {0x224c, 0,  0xac},
  {0x224d, 17, 0xde},
  {0x224e, 17, 0xdb},
  {0x224f, 17, 0x22},
  {0x2250, 17, 0x24},
  {0x225c, 17, 0xb0},
  {0x225d, 17, 0x5e},
  {0x225e, 0, RAW_HERSHEY_GLYPH + 4008},
  {0x225f, 17, 0xb6},
  {0x2260, 17, 0xd1},
  {0x2261, 17, 0xba},
  {0x2262, 0, RAW_HERSHEY_GLYPH + 276},
  /* 0x2263..0x2264 are \ll and \gg */
  {0x2265, 17, 0xd6},
  /* 0x2266 is an inverted Hershey 765 `glaze' glyph */
  {0x2267, 17, 0xb5},
  /* 0x2268 is an inverted `therefore' */
  {0x2269, 17, 0xf2},
  /* 0x2270 is a double integral */
  {0x2272, 0,  0xc5},
  /* 0x2273 is perthousand */
  {0x2274, 0,  RAW_HERSHEY_GLYPH + 2323},
  {0x2275, 0,  RAW_HERSHEY_GLYPH + 2325},
  /* 0x2276 is an eighth note */
  {0x2277, 0,  RAW_HERSHEY_GLYPH + 2277},
  {0x2278, 0,  RAW_HERSHEY_GLYPH + 2278},
  {0x2279, 0,  0xb6},
  {0x227e, 0,  RAW_HERSHEY_GLYPH + 905},
/* Row 3 [Roman characters], taken from HersheySerif (font #0) */
  {0x2330, 0, '0'},
  {0x2331, 0, '1'},
  {0x2332, 0, '2'},
  {0x2333, 0, '3'},
  {0x2334, 0, '4'},
  {0x2335, 0, '5'},
  {0x2336, 0, '6'},
  {0x2337, 0, '7'},
  {0x2338, 0, '8'},
  {0x2339, 0, '9'},
  {0x2341, 0, 'A'},    
  {0x2342, 0, 'B'},    
  {0x2343, 0, 'C'},    
  {0x2344, 0, 'D'},    
  {0x2345, 0, 'E'},    
  {0x2346, 0, 'F'},    
  {0x2347, 0, 'G'},    
  {0x2348, 0, 'H'},    
  {0x2349, 0, 'I'},    
  {0x234a, 0, 'J'},    
  {0x234b, 0, 'K'},    
  {0x234c, 0, 'L'},    
  {0x234d, 0, 'M'},    
  {0x234e, 0, 'N'},    
  {0x234f, 0, 'O'},    
  {0x2350, 0, 'P'},    
  {0x2351, 0, 'Q'},    
  {0x2352, 0, 'R'},    
  {0x2353, 0, 'S'},    
  {0x2354, 0, 'T'},    
  {0x2355, 0, 'U'},    
  {0x2356, 0, 'V'},    
  {0x2357, 0, 'W'},    
  {0x2358, 0, 'X'},    
  {0x2359, 0, 'Y'},    
  {0x235a, 0, 'Z'},    
  {0x2361, 0, 'a'},    
  {0x2362, 0, 'b'},    
  {0x2363, 0, 'c'},    
  {0x2364, 0, 'd'},    
  {0x2365, 0, 'e'},    
  {0x2366, 0, 'f'},    
  {0x2367, 0, 'g'},    
  {0x2368, 0, 'h'},    
  {0x2369, 0, 'i'},    
  {0x236a, 0, 'j'},    
  {0x236b, 0, 'k'},    
  {0x236c, 0, 'l'},    
  {0x236d, 0, 'm'},    
  {0x236e, 0, 'n'},    
  {0x236f, 0, 'o'},    
  {0x2370, 0, 'p'},    
  {0x2371, 0, 'q'},    
  {0x2372, 0, 'r'},    
  {0x2373, 0, 's'},    
  {0x2374, 0, 't'},    
  {0x2375, 0, 'u'},    
  {0x2376, 0, 'v'},    
  {0x2377, 0, 'w'},    
  {0x2378, 0, 'x'},    
  {0x2379, 0, 'y'},    
  {0x237a, 0, 'z'},    
/* Row 4 [Hiragana], taken from HersheyHiragana (font #14) */
  {0x2421, 14, 0x21},
  {0x2422, 14, 0x22},
  {0x2423, 14, 0x23},
  {0x2424, 14, 0x24},
  {0x2425, 14, 0x25},
  {0x2426, 14, 0x26},
  {0x2427, 14, 0x27},
  {0x2428, 14, 0x28},
  {0x2429, 14, 0x29},
  {0x242a, 14, 0x2a},
  {0x242b, 14, 0x2b},
  {0x242c, 14, 0x2c},
  {0x242d, 14, 0x2d},
  {0x242e, 14, 0x2e},
  {0x242f, 14, 0x2f},
  {0x2430, 14, 0x30},
  {0x2431, 14, 0x31},
  {0x2432, 14, 0x32},
  {0x2433, 14, 0x33},
  {0x2434, 14, 0x34},
  {0x2435, 14, 0x35},
  {0x2436, 14, 0x36},
  {0x2437, 14, 0x37},
  {0x2438, 14, 0x38},
  {0x2439, 14, 0x39},
  {0x243a, 14, 0x3a},
  {0x243b, 14, 0x3b},
  {0x243c, 14, 0x3c},
  {0x243d, 14, 0x3d},
  {0x243e, 14, 0x3e},
  {0x243f, 14, 0x3f},
  {0x2440, 14, 0x40},
  {0x2441, 14, 0x41},
  {0x2442, 14, 0x42},
  {0x2443, 14, 0x43},
  {0x2444, 14, 0x44},
  {0x2445, 14, 0x45},
  {0x2446, 14, 0x46},
  {0x2447, 14, 0x47},
  {0x2448, 14, 0x48},
  {0x2449, 14, 0x49},
  {0x244a, 14, 0x4a},
  {0x244b, 14, 0x4b},
  {0x244c, 14, 0x4c},
  {0x244d, 14, 0x4d},
  {0x244e, 14, 0x4e},
  {0x244f, 14, 0x4f},
  {0x2450, 14, 0x50},
  {0x2451, 14, 0x51},
  {0x2452, 14, 0x52},
  {0x2453, 14, 0x53},
  {0x2454, 14, 0x54},
  {0x2455, 14, 0x55},
  {0x2456, 14, 0x56},
  {0x2457, 14, 0x57},
  {0x2458, 14, 0x58},
  {0x2459, 14, 0x59},
  {0x245a, 14, 0x5a},
  {0x245b, 14, 0x5b},
  {0x245c, 14, 0x5c},
  {0x245d, 14, 0x5d},
  {0x245e, 14, 0x5e},
  {0x245f, 14, 0x5f},
  {0x2460, 14, 0x60},
  {0x2461, 14, 0x61},
  {0x2462, 14, 0x62},
  {0x2463, 14, 0x63},
  {0x2464, 14, 0x64},
  {0x2465, 14, 0x65},
  {0x2466, 14, 0x66},
  {0x2467, 14, 0x67},
  {0x2468, 14, 0x68},
  {0x2469, 14, 0x69},
  {0x246a, 14, 0x6a},
  {0x246b, 14, 0x6b},
  {0x246c, 14, 0x6c},
  {0x246d, 14, 0x6d},
  {0x246e, 14, 0x6e},
  {0x246f, 14, 0x6f},
  {0x2470, 14, 0x70},
  {0x2471, 14, 0x71},
  {0x2472, 14, 0x72},
  {0x2473, 14, 0x73},
/* Row 5 [Katakana], taken from HersheyKatakana (font #15) */
  {0x2521, 15, 0x21},
  {0x2522, 15, 0x22},
  {0x2523, 15, 0x23},
  {0x2524, 15, 0x24},
  {0x2525, 15, 0x25},
  {0x2526, 15, 0x26},
  {0x2527, 15, 0x27},
  {0x2528, 15, 0x28},
  {0x2529, 15, 0x29},
  {0x252a, 15, 0x2a},
  {0x252b, 15, 0x2b},
  {0x252c, 15, 0x2c},
  {0x252d, 15, 0x2d},
  {0x252e, 15, 0x2e},
  {0x252f, 15, 0x2f},
  {0x2530, 15, 0x30},
  {0x2531, 15, 0x31},
  {0x2532, 15, 0x32},
  {0x2533, 15, 0x33},
  {0x2534, 15, 0x34},
  {0x2535, 15, 0x35},
  {0x2536, 15, 0x36},
  {0x2537, 15, 0x37},
  {0x2538, 15, 0x38},
  {0x2539, 15, 0x39},
  {0x253a, 15, 0x3a},
  {0x253b, 15, 0x3b},
  {0x253c, 15, 0x3c},
  {0x253d, 15, 0x3d},
  {0x253e, 15, 0x3e},
  {0x253f, 15, 0x3f},
  {0x2540, 15, 0x40},
  {0x2541, 15, 0x41},
  {0x2542, 15, 0x42},
  {0x2543, 15, 0x43},
  {0x2544, 15, 0x44},
  {0x2545, 15, 0x45},
  {0x2546, 15, 0x46},
  {0x2547, 15, 0x47},
  {0x2548, 15, 0x48},
  {0x2549, 15, 0x49},
  {0x254a, 15, 0x4a},
  {0x254b, 15, 0x4b},
  {0x254c, 15, 0x4c},
  {0x254d, 15, 0x4d},
  {0x254e, 15, 0x4e},
  {0x254f, 15, 0x4f},
  {0x2550, 15, 0x50},
  {0x2551, 15, 0x51},
  {0x2552, 15, 0x52},
  {0x2553, 15, 0x53},
  {0x2554, 15, 0x54},
  {0x2555, 15, 0x55},
  {0x2556, 15, 0x56},
  {0x2557, 15, 0x57},
  {0x2558, 15, 0x58},
  {0x2559, 15, 0x59},
  {0x255a, 15, 0x5a},
  {0x255b, 15, 0x5b},
  {0x255c, 15, 0x5c},
  {0x255d, 15, 0x5d},
  {0x255e, 15, 0x5e},
  {0x255f, 15, 0x5f},
  {0x2560, 15, 0x60},
  {0x2561, 15, 0x61},
  {0x2562, 15, 0x62},
  {0x2563, 15, 0x63},
  {0x2564, 15, 0x64},
  {0x2565, 15, 0x65},
  {0x2566, 15, 0x66},
  {0x2567, 15, 0x67},
  {0x2568, 15, 0x68},
  {0x2569, 15, 0x69},
  {0x256a, 15, 0x6a},
  {0x256b, 15, 0x6b},
  {0x256c, 15, 0x6c},
  {0x256d, 15, 0x6d},
  {0x256e, 15, 0x6e},
  {0x256f, 15, 0x6f},
  {0x2570, 15, 0x70},
  {0x2571, 15, 0x71},
  {0x2572, 15, 0x72},
  {0x2573, 15, 0x73},
/* Row 6 [Greek characters], taken from HersheySerif-Symbol (font #17) */
  {0x2621, 17, 'A'},    
  {0x2622, 17, 'B'},    
  {0x2623, 17, 'G'},    
  {0x2624, 17, 'D'},    
  {0x2625, 17, 'E'},    
  {0x2626, 17, 'Z'},    
  {0x2627, 17, 'H'},    
  {0x2628, 17, 'Q'},    
  {0x2629, 17, 'I'},    
  {0x262a, 17, 'K'},    
  {0x262b, 17, 'L'},    
  {0x262c, 17, 'M'},    
  {0x262d, 17, 'N'},    
  {0x262e, 17, 'X'},    
  {0x262f, 17, 'O'},    
  {0x2630, 17, 'P'},    
  {0x2631, 17, 'R'},    
  {0x2632, 17, 'S'},    
  {0x2633, 17, 'T'},    
  {0x2634, 17, 0x80 + '!'},	/* variant upsilon */
  {0x2635, 17, 'F'},    
  {0x2636, 17, 'C'},    
  {0x2637, 17, 'Y'},    
  {0x2638, 17, 'W'},    
  {0x2641, 17, 'a'},    
  {0x2642, 17, 'b'},    
  {0x2643, 17, 'g'},    
  {0x2644, 17, 'd'},    
  {0x2645, 17, 'e'},    
  {0x2646, 17, 'z'},    
  {0x2647, 17, 'h'},    
  {0x2648, 17, 'q'},    
  {0x2649, 17, 'i'},    
  {0x264a, 17, 'k'},    
  {0x264b, 17, 'l'},    
  {0x264c, 17, 'm'},    
  {0x264d, 17, 'n'},    
  {0x264e, 17, 'x'},    
  {0x264f, 17, 'o'},    
  {0x2650, 17, 'p'},    
  {0x2651, 17, 'r'},    
  {0x2652, 17, 's'},    
  {0x2653, 17, 't'},    
  {0x2654, 17, 'u'},
  {0x2655, 17, 'f'},    
  {0x2656, 17, 'c'},    
  {0x2657, 17, 'y'},    
  {0x2658, 17, 'w'},    
/* Row 7 [Cyrillic characters], taken from HersheyCyrillic (font #13).
   The strange order is because we are mapping to the KOI8-R encoding. */
  {0x2721, 13, 0x80 + 'a'},
  {0x2722, 13, 0x80 + 'b'}, 
  {0x2723, 13, 0x80 + 'w'},
  {0x2724, 13, 0x80 + 'g'},
  {0x2725, 13, 0x80 + 'd'},
  {0x2726, 13, 0x80 + 'e'},
  {0x2727, 13, 0x80 + '3'},
  {0x2728, 13, 0x80 + 'v'},
  {0x2729, 13, 0x80 + 'z'},
  {0x272a, 13, 0x80 + 'i'},
  {0x272b, 13, 0x80 + 'j'},
  {0x272c, 13, 0x80 + 'k'},
  {0x272d, 13, 0x80 + 'l'},
  {0x272e, 13, 0x80 + 'm'},
  {0x272f, 13, 0x80 + 'n'},
  {0x2730, 13, 0x80 + 'o'},
  {0x2731, 13, 0x80 + 'p'},
  {0x2732, 13, 0x80 + 'r'}, 
  {0x2733, 13, 0x80 + 's'},
  {0x2734, 13, 0x80 + 't'},
  {0x2735, 13, 0x80 + 'u'},
  {0x2736, 13, 0x80 + 'f'},
  {0x2737, 13, 0x80 + 'h'},
  {0x2738, 13, 0x80 + 'c'},
  {0x2739, 13, 0x80 + '~'},
  {0x273a, 13, 0x80 + '{'},
  {0x273b, 13, 0x80 + '}'},
  {0x273c, 13, 0x80 + 0x7f},
  {0x273d, 13, 0x80 + 'y'},
  {0x273e, 13, 0x80 + 'x'},
  {0x273f, 13, 0x80 + '|'},
  {0x2740, 13, 0x80 + '`'},
  {0x2741, 13, 0x80 + 'q'},
  {0x2751, 13, 0x80 + 'A'},
  {0x2752, 13, 0x80 + 'B'}, 
  {0x2753, 13, 0x80 + 'W'},
  {0x2754, 13, 0x80 + 'G'},
  {0x2755, 13, 0x80 + 'D'},
  {0x2756, 13, 0x80 + 'E'},
  {0x2757, 13, 0x80 + '#'},
  {0x2758, 13, 0x80 + 'V'},
  {0x2759, 13, 0x80 + 'Z'},
  {0x275a, 13, 0x80 + 'I'},
  {0x275b, 13, 0x80 + 'J'},
  {0x275c, 13, 0x80 + 'K'},
  {0x275d, 13, 0x80 + 'L'},
  {0x275e, 13, 0x80 + 'M'},
  {0x275f, 13, 0x80 + 'N'},
  {0x2760, 13, 0x80 + 'O'},
  {0x2761, 13, 0x80 + 'P'},
  {0x2762, 13, 0x80 + 'R'}, 
  {0x2763, 13, 0x80 + 'S'},
  {0x2764, 13, 0x80 + 'T'},
  {0x2765, 13, 0x80 + 'U'},
  {0x2766, 13, 0x80 + 'F'},
  {0x2767, 13, 0x80 + 'H'},
  {0x2768, 13, 0x80 + 'C'},
  {0x2769, 13, 0x80 + '^'},
  {0x276a, 13, 0x80 + '['},
  {0x276b, 13, 0x80 + ']'},
  {0x276c, 13, 0x80 + '_'},
  {0x276d, 13, 0x80 + 'Y'},
  {0x276e, 13, 0x80 + 'X'},
  {0x276f, 13, 0x80 + '\\'},
  {0x2770, 13, 0x80 + '@'},
  {0x2771, 13, 0x80 + 'Q'},
  {0, 0, 0}
};
